//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "CreateScenario.h"

#include "counter.h"
#include "real_player.h"
#include "ai_fast.h"
#include "ai_dummy.h"

CreateScenario::CreateScenario()
    :d_generator(0), d_xsize(100), d_ysize(100)
{
    d_mapdef = new Mapdef();
    d_playerlist = new Playerlist();
    d_armysetlist = new Armysetlist();
    d_stacklist = new Stacklist();
    d_ruinlist = new Ruinlist();
    d_citylist = new Citylist;
    d_templelist = new TempleList();
    fl_counter = new FL_Counter();
}

CreateScenario::~CreateScenario()
{
    if (d_generator)
        delete d_generator;

    delete d_mapdef;
    delete d_playerlist;
    delete d_armysetlist;
    delete d_stacklist;
    delete d_ruinlist;
    delete d_citylist;
    delete d_templelist;
    delete fl_counter;
}

void CreateScenario::setMaptype(MapType type)
{
    //this function currently does almost nothing. It's purpose is to initialize
    //the right MapGenerator (e.g. islands, mountain chains, normal etc.).
    //Currently we have only a normal d_generator, so it is initialized here

    if (d_generator)
        delete d_generator;

    d_generator = new MapGenerator();
}

void CreateScenario::setPercentages(int pwater, int pforest, int pswamp,
                                    int phills, int pmountains)
{
    if (!d_generator)
        setMaptype(NORMAL);

    d_generator->setPercentages(pwater, pforest, pswamp, phills, pmountains);
}

void CreateScenario::setNoCities(int nocities)
{
    if (!d_generator)
        setMaptype(NORMAL);

    d_generator->setNoCities(nocities);
}

void CreateScenario::setNoRuins(int noruins)
{
    if (!d_generator)
        setMaptype(NORMAL);

    d_generator->setNoRuins(noruins);
}

void CreateScenario::setNoTemples(int notemples)
{
    if (!d_generator)
        setMaptype(NORMAL);

    d_generator->setNoTemples(notemples);
}

void CreateScenario::setSize(int xsize, int ysize)
{
    if ((xsize < 0) || (ysize < 0))
    {
        std::cerr <<"CreateScenario:: wrong size given\n";
        return;
    }

    d_xsize = xsize;
    d_ysize = ysize;
}

void CreateScenario::addPlayer(Playerstruct* ps)
{
    Player* p;

    switch (ps->type)
    {
        case Player::HUMAN:
            p = new RealPlayer(ps->name, ps->armyset, ps->color);
            break;
        case Player::AI_FAST:
            p = new AI_Fast(ps->name, ps->armyset, ps->color);
            break;
        case Player::AI_DUMMY:
            p = new AI_Dummy(ps->name, ps->armyset, ps->color);
            break;
        case Player::NEUTRAL:
            p = new AI_Dummy(ps->name, ps->armyset, ps->color);
            d_playerlist->setNeutral(p);
            break;
        default:
            std::cerr <<"CreateScenario: wrong player type\n";
    }

    d_playerlist->push_back(p);
}

int CreateScenario::getNoPlayers()
{
    return d_playerlist->size();
}

Player* CreateScenario::getPlayer(int number)
{
    if (number >= static_cast<int>(d_playerlist->size()))
        return 0;

    Playerlist::iterator it;
    for (it = d_playerlist->begin(); number > 0; number--)
        it++;

    return (*it);
}

CreateScenario::MapType CreateScenario::getMaptype()
{
    //there is only one maptype yet
    return NORMAL;
}

int CreateScenario::getNoCities()
{
    if (!d_generator)
        return 0;

    return d_generator->getNoCities();
}

int CreateScenario::getNoRuins()
{
    if (!d_generator)
        return 0;

    return d_generator->getNoRuins();
}

int CreateScenario::getNoTemples()
{
    if (!d_generator)
        return 0;

    return d_generator->getNoTemples();
}

bool CreateScenario::create()
{
    if (!createMap())
        return false;

    if (!distributePlayers())
        return false;

    if (!setupCities())
        return false;

    if (!setupRuins())
        return false;

    if (!setupPlayers())
        return false;

    return true;
}

bool CreateScenario::dump()
{
    //this function basically imitates the save function of GameScenario.
    //So, if the savegame style changes, these changes have to be done here, too
    const std::string randommapfilename = "random.map";
    bool retval = true;

    XML_Helper* helper = new XML_Helper(randommapfilename, std::ios::out);
    retval &= helper->begin(FREELORDS_SAVEGAME_VERSION);
    retval &= helper->openTag("freelords");

    retval &= helper->saveData("name", "random scenario");
    retval &= helper->saveData("type", GameScenario::SINGLE_PLAYER);
    retval &= helper->saveData("turn", 0);
    retval &= helper->closeTag();

    //now save the rest of the data
    retval &= fl_counter->save(helper);

    retval &= d_mapdef->save(helper);
    retval &= d_playerlist->save(helper);
    retval &= d_citylist->save(helper);
    retval &= d_templelist->save(helper);
    retval &= d_ruinlist->save(helper);
    retval &= d_stacklist->save(helper);

    retval &= helper->closeTag();
    
    helper->close();
    delete helper;

    return retval;
}

bool CreateScenario::createMap()
{
    int temp;
    char type;
    char* map;
    
    //have the generator make the map...
    d_generator->MakeMap(d_xsize, d_ysize);
    
    //...fill the terrain...
    d_mapdef->fill(d_generator);

    //...and create cities, temples and ruins
    map = d_generator->getBuildings(temp, temp);
    
    for (int j = 0; j < d_ysize; j++)
        for (int i = 0; i < d_xsize; i++)
        {
            type = map[i + j*d_xsize];
            PG_Point pos;
            pos.x = i;      pos.y = j;

            switch (type)
            {
                case CHARACTER_TEMPLE:
                    d_templelist->push_back(Temple(pos));
                    break;
                case CHARACTER_RUIN:
                    d_ruinlist->push_back(Ruin(pos));
                    break;
                case CHARACTER_CITY_MAJOR:
                    d_citylist->push_back(City(pos));
            }
        }
    //the other details such as giving names are done later
    
    return true;
}

bool CreateScenario::distributePlayers()
{
   //how many cities we skip after a player assignment
    int cityskip = (d_citylist->size() - 1) / (d_playerlist->size() - 1) - 1;

    //some variables
    int skipping = 0;
    Playerlist::iterator pit = d_playerlist->begin();
    if ((*pit) == d_playerlist->getNeutral())
        pit++;

    for (Citylist::iterator cit = d_citylist->begin(); cit != d_citylist->end();
        cit++)
    {
        skipping++;
        if ((skipping >= cityskip) && (pit != d_playerlist->end()))
        {
            (*cit).setPlayer(*pit);
            pit++;
        
            if ((*pit) == d_playerlist->getNeutral())
                pit++;
        }
        else
            (*cit).setPlayer(d_playerlist->getNeutral());
    }

    return true;
}

bool CreateScenario::setupCities()
{
    if (!loadCitynames())
    {
        std::cerr <<"Couldn't load citynames. Aborting scenario creation.\n";
        return false;
    }

    for (Citylist::iterator it = d_citylist->begin(); it != d_citylist->end();
        it++)
    {
        //1. set a reasonable cityname
        int randno;
        
        randno = rand() % d_citynames.size();
        (*it).setName(d_citynames[randno]);
        //remove the used name
        d_citynames[randno] = d_citynames[d_citynames.size() - 1];
        d_citynames.pop_back();

        //2. distribute the income a bit (not done here)

        //3. set the city production
        (*it).setRandomArmytypes();
        (*it).produceLargestArmy();
        if ((*it).getPlayer() == d_playerlist->getNeutral())
            (*it).setProduction(-1);
        else
            (*it).setProduction(0);
    }

    return true;
}

bool CreateScenario::setupRuins()
{
    return true;
}

bool CreateScenario::setupPlayers()
{
    return true;
}

bool CreateScenario::loadCitynames()
{
    return true;
}
