//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "MapRenderer.h"
#include "mapdef.h"
#include <SDL.h>
#include <string>

using namespace std;

MapRenderer::MapRenderer(SDL_Surface* surface, Mapdef* mapdef)
{
    d_surface = surface;
    d_mapdef = mapdef;
}

MapRenderer::~MapRenderer()
{
}

// set transitions

int MapRenderer::smoothCorner1(bool lt, bool rt, bool lb)
{
    if (lt && rt && lb) return 0;
    if (lb && !rt) return 4;
    if (rt && !lb) return 8;
    if (!rt && !lb) return 12;
    if (!lt && rt && lb) return 16;
    return 0;
}

int MapRenderer::smoothCorner2(bool lt, bool rt, bool rb)
{
    if (lt && rt && rb) return 1;
    if (!lt && rb) return 5;
    if (lt && !rb) return 9;
    if (!lt && !rb) return 13;
    if (lt && !rt && rb) return 17;
    return 1;
}

int MapRenderer::smoothCorner3(bool lt, bool lb, bool rb)
{
    if (lt && lb && rb) return 2;
    if (lt && !rb) return 6;
    if (!lt && rb) return 10;
    if (!lt && !rb) return 14;
    if (lt && !lb && rb) return 18;
    return 2;
}

int MapRenderer::smoothCorner4(bool rt, bool lb, bool rb)
{
    if (rt && lb && rb) return 3;
    if (rt && !lb) return 7;
    if (!rt && lb) return 11;
    if (!rt && !lb) return 15;
    if (rt && lb && !rb) return 19;
    return 3;
}

void MapRenderer::smooth(int x, int y)
{
    if (d_mapdef->map[x][y]->isSmoothed()) return;
    int basetile = d_mapdef->map[x][y]->getType();
    bool l = true;
    bool r = true;
    bool t = true;
    bool b = true;
    bool lt = true;
    bool rt = true;
    bool lb = true;
    bool rb = true;

    // flags are set to true if tile is the same
    if (x-1 >= 0) l = (basetile == d_mapdef->map[x-1][y]->getType());
    if (x+1 <= 99) r = (basetile == d_mapdef->map[x+1][y]->getType());
    if (y-1 >= 0) t = (basetile == d_mapdef->map[x][y-1]->getType());
    if (y+1 <= 99) b = (basetile == d_mapdef->map[x][y+1]->getType());
    if (x-1 >= 0 && y-1 >= 0) 
        lt = (basetile == d_mapdef->map[x-1][y-1]->getType());
    if (x+1 <= 99 && y-1 >= 0) 
        rt = (basetile == d_mapdef->map[x+1][y-1]->getType());
    if (x-1 >= 0 && y+1 <= 99) 
        lb = (basetile == d_mapdef->map[x-1][y+1]->getType());
    if (x+1 <= 99 && y+1 <= 99) 
        rb = (basetile == d_mapdef->map[x+1][y+1]->getType());

    int corner[4];
    corner[0] = smoothCorner1(lt, t, l);
    corner[1] = smoothCorner2(t, rt, r);
    corner[2] = smoothCorner3(l, lb, b);
    corner[3] = smoothCorner4(r, b, rb);

    // use variation tiles if the result it the basetile
    if (corner[0] == 0 && corner[1] == 1 && corner[2] == 2 && corner[3] == 3)
    {
        // use one of the possible 3 variations
        int var = rand() % 4;
        // if var == 0 use base tile => no change needed
        if (var > 0)
        {
            for (int i = 0; i < 4; i++)
            {
                // variations start at corner 20
                corner[i] += 20 + (var-1)*4;
            }
        }
    }

    // set calculated corner tiles
    d_mapdef->map[x][y]->setCorners(corner[0], corner[1], corner[2], corner[3]);
}

void MapRenderer::getCornerRect(SDL_Rect& rect, int i)
{
    rect.x = 0; 
    rect.y = 0;
    if (i == 1 || i == 3) rect.x = 25; 
    if (i == 2 || i == 4) rect.y = 25; 
}

void MapRenderer::render(int x, int y, int tileStartX, int tileStartY,
        int columns, int rows)
{
    SDL_Rect r;
    r.w = r.h = d_mapdef->getTileSet()->getTileSize() / 2;
    int drawY = y;
    for(int tileY = tileStartY; tileY < (tileStartY + rows); tileY++)
    {
        int drawX = x;
        for(int tileX = tileStartX; tileX < (tileStartX + columns); tileX++)
        {
            // smooth it
            smooth(tileX,tileY);

            // get correct tile
            Tile* type = d_mapdef->getTileSet()->getTile(
                    d_mapdef->map[tileX][tileY]->getType());

            // render all 4 corners
            r.x = drawX;
            r.y = drawY;
            int corner = d_mapdef->map[tileX][tileY]->getCorner(0);
            SDL_BlitSurface(type->getSurface(corner), 0, d_surface, &r);

            r.x = drawX+r.w;
            r.y = drawY;
            corner = d_mapdef->map[tileX][tileY]->getCorner(1);
            SDL_BlitSurface(type->getSurface(corner), 0, d_surface, &r);

            r.x = drawX;
            r.y = drawY+r.h;
            corner = d_mapdef->map[tileX][tileY]->getCorner(2);
            SDL_BlitSurface(type->getSurface(corner), 0, d_surface, &r);

            r.x = drawX + r.w;
            r.y = drawY + r.h;
            corner = d_mapdef->map[tileX][tileY]->getCorner(3);
            SDL_BlitSurface(type->getSurface(corner), 0, d_surface, &r);

            drawX += d_mapdef->getTileSet()->getTileSize();
        }
        drawY += d_mapdef->getTileSet()->getTileSize();
    }					
}

// End of file
